@@ -407,6 +407,7 @@ harvest_committables(apr_hash_t *committables,
   const char *cf_url = NULL;
   svn_revnum_t cf_rev = entry->copyfrom_rev;
   const svn_string_t *propval;
+  svn_boolean_t matches_changelists;
   svn_boolean_t is_special;
   apr_pool_t *token_pool = (lock_tokens ? apr_hash_pool_get(lock_tokens)
                             : NULL);
@@ -434,6 +435,16 @@ harvest_committables(apr_hash_t *committables,
 
   SVN_ERR(svn_io_check_special_path(path, &kind, &is_special, scratch_pool));
 
+  /* Save the result for reuse. */
+  matches_changelists = SVN_WC__CL_MATCH(changelists, entry);
+
+  /* Early exit. */
+  if (kind != svn_node_dir && kind != svn_node_none
+      && ! matches_changelists)
+    {
+      return SVN_NO_ERROR;
+    }
+
   if ((kind != svn_node_file)
       && (kind != svn_node_dir)
       && (kind != svn_node_none))
@@ -1632,6 +1643,7 @@ svn_client__do_commit(const char *base_url,
       unsigned char digest[APR_MD5_DIGESTSIZE];
       svn_boolean_t fulltext = FALSE;
       svn_wc_adm_access_t *item_access;
+      svn_error_t *err;
 
       svn_pool_clear(iterpool);
       /* Get the next entry. */
@@ -1660,12 +1672,23 @@ svn_client__do_commit(const char *base_url,
         fulltext = TRUE;
 
       dir_path = svn_path_dirname(item->path, iterpool);
-      SVN_ERR(svn_wc_adm_retrieve(&item_access, adm_access, dir_path,
-                                  iterpool));
-      SVN_ERR(svn_wc_transmit_text_deltas2(tempfiles ? &tempfile : NULL,
-                                           digest, item->path,
-                                           item_access, fulltext, editor,
-                                           file_baton, iterpool));
+      err = svn_wc_adm_retrieve(&item_access, adm_access, dir_path,
+                                iterpool);
+      if (err)
+        {
+          svn_pool_destroy(iterpool);
+          return err;
+        }
+      err = svn_wc_transmit_text_deltas2(tempfiles ? &tempfile : NULL,
+                                         digest, item->path,
+                                         item_access, fulltext, editor,
+                                         file_baton, iterpool);
+      if (err)
+        {
+          svn_pool_destroy(iterpool);
+          return err;
+        }
+
       if (tempfiles && tempfile)
         {
           tempfile = apr_pstrdup(pool, tempfile);
